package service_award_dividend

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/service_award_dividend/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/implement"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models"
	"errors"
	"fmt"
	"math/rand"
	"time"
	"xorm.io/xorm"
)

const ResetServiceAwardDividendRelationLockKey = "reset_service_award_dividend_relation_lock_key"
const ResetServiceAwardDividendRelationLockValue = "running"

func ResetServiceAwardDividendRelation(ch *rabbit.Channel, engine *xorm.Engine, dbName string) (err error) {
	fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!ResetServiceAwardDividendRelation----->dbName!!!!!!!!!!!!!!!!!!!!", dbName)

	//TODO::增加“悲观锁”防止串行
	getString, _ := cache.GetString(ResetServiceAwardDividendRelationLockKey)
	//if err != nil {
	//	return err
	//}
	if getString == ResetServiceAwardDividendRelationLockValue {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次结算未执行完")
		return errors.New("上一次结算未执行完")
	}
	cache.SetEx(ResetServiceAwardDividendRelationLockKey, ResetServiceAwardDividendRelationLockValue, 3600*12) //12小时

	//1、查找 `service_award_dividend_basic` 基础设置
	serviceAwardDividendBasicDb := implement.NewServiceAwardDividendBasicDb(engine)
	base, _ := serviceAwardDividendBasicDb.GetServiceAwardDividendBasic()
	if base == nil {
		err = errors.New("服务奖分红未设置！")
		return
	}
	if base.IsOpen != 1 {
		err = errors.New("服务奖分红未开启！")
		return
	}
	if base.IsRefreshAllocation != 1 {
		err = errors.New("服务奖分红未开启重新分配！")
		return
	}
	now := time.Now()
	today := now.Format("2006-01-02")
	if base.RefreshAllocation != today {
		return errors.New("非重新分配时间！")
	}

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", now.Hour())
	if now.Hour() > 1 && now.Hour() < 8 {
		//TODO::只在凌晨一点 ~ 凌晨 8 点运行
		return errors.New("非运行时间")
	}

	//2、将现有数据全部推至 RabbitMq 消费
	var page = 1
	var pageSize = 1000
	for {
		var list []models.ServiceAwardDividendRelation
		err = engine.Limit(pageSize, (page-1)*pageSize).Asc("id").Find(&list)
		if err != nil {
			fmt.Println("err:::::1111", err)
			return
		}
		if len(list) <= 0 {
			break
		}
		for _, v := range list {
			//TODO::推入rabbitmq 异步处理
			ch.Publish(md.ServiceAwardDividendExchange, md.JoinServiceAwardDividendRelationReq{
				MasterId:  dbName,
				Uid:       v.Uid,
				ParentUid: v.ParentUid,
				RegionId:  v.RegionId,
				CreateAt:  v.CreateAt,
			}, md.ServiceAwardDividendRoutKeyForRelation)
		}
	}

	//3、进行数据清理
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()

	sql1 := "DROP TABLE `service_award_dividend_relation_1`"                                                 //删除备份表
	sql2 := "DROP TABLE `service_award_dividend_relation_tmp`"                                               //删除临时表
	sql3 := "CREATE TABLE `service_award_dividend_relation_tmp` LIKE `service_award_dividend_relation`"      //复制表结构
	sql4 := "ALTER TABLE `service_award_dividend_relation` RENAME TO `service_award_dividend_relation_1`;"   //将现有表重命名 -> 备份表
	sql5 := "ALTER TABLE `service_award_dividend_relation_tmp` RENAME TO `service_award_dividend_relation`;" //将临时表重命名 -> 现有表
	_, err = db.ExecuteOriginalSqlBySession(engine.NewSession(), sql1)
	if err != nil {
		_ = session.Rollback()
		fmt.Println("err___SQl1", err.Error())
	}
	time.Sleep(time.Millisecond * time.Duration(rand.Intn(1000)))
	_, err = db.ExecuteOriginalSqlBySession(engine.NewSession(), sql2)
	if err != nil {
		_ = session.Rollback()
		fmt.Println("err___SQl2", err.Error())
	}

	time.Sleep(time.Millisecond * time.Duration(rand.Intn(1000)))
	_, err = db.ExecuteOriginalSqlBySession(engine.NewSession(), sql3)
	if err != nil {
		_ = session.Rollback()
		return err
	}
	time.Sleep(time.Millisecond * time.Duration(rand.Intn(1000)))
	_, err = db.ExecuteOriginalSqlBySession(engine.NewSession(), sql4)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	time.Sleep(time.Millisecond * time.Duration(rand.Intn(1000)))
	_, err = db.ExecuteOriginalSqlBySession(engine.NewSession(), sql5)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return err
	}

	return
}
